@@ -95,4 +95,58 @@ def set_status_bar_message(text: str, hide_after_timeout: int = None) -> Disposa
         end="",
     )
     res = uinput()
-    return Disposable(res)
\ No newline at end of file
+    return Disposable(res)
+
+class ActiveTextEditor:
+    """
+    The currently active editor or undefined. The active editor is the one that currently has focus or, when none has focus, the one that has changed input most recently.
+    """
+
+    def __init__(self):
+        print('AT', flush=True, end="")
+        res = uinput()
+        if not res:
+            self = undefined
+        res = json.loads(res.replace(r'\\', r'\\\\'))
+        self.__dict__.update(apply_func_to_keys(res, camel_to_snake))
+        self.document = TextDocument(self.document)
+        
+class Range:
+    "A range represents an ordered pair of two positions. It is guaranteed that start.isBeforeOrEqual(end)"
+    def __init__(self, data):
+        self.__dict__.update(data)
+
+    # TODO: Range methods
+
+class Position:
+    """
+    Represents a line and character position, such as the position of the cursor.
+    """
+    def __init__(self, data):
+        self.__dict__.update(data)
+
+    # TODO: Position methods
+
+class TextDocument:
+    """
+    Represents a text document, such as a source file. Text documents have lines and knowledge about an underlying resource like a file.
+    """
+    def __init__(self, data):
+        self.__dict__.update(data)
+
+    def get_text(self, range: Range = None) -> str:
+        """
+        Get the text of this document. A substring can be retrieved by providing a range. The range will be adjusted.
+        """
+        if range is not None:
+            if isinstance(range, Range):
+                range = range.__dict__
+            range = json.dumps(range)
+            print(f'GT: {range}', flush=True, end="")
+        else:
+            print('GT', flush=True, end="")
+
+        return uinput()
+    
+    # TODO: TextDocument methods
+
